@@ -214,11 +214,13 @@ gcal_application_show_about (GSimpleAction *simple,
     "Erick Pérez Castellanos <erickpc@gnome.org>",
     "Georges Basile Stavracas Neto <georges.stavracas@gmail.com>",
     "Isaque Galdino <igaldino@gmail.com>",
+    "Titouan Real <titouan.real@gmail.com>",
     NULL
   };
   const gchar *designers[] = {
     "Jakub Steiner <jimmac@gmail.com>",
     "Lapo Calamandrei <calamandrei@gmail.com>",
+    "Philipp Sauberzweig",
     "Reda Lazri <the.red.shortcut@gmail.com>",
     "William Jon McCann <jmccann@redhat.com>",
     NULL
