@@ -53,9 +53,9 @@ struct _GcalEventWidget
 
   /* widgets */
   GtkWidget          *horizontal_box;
-  GtkWidget          *main_widget;
   GtkWidget          *timestamp_label;
-  GtkWidget          *squeezer;
+  GtkWidget          *edge;
+  GtkWidget          *overflow_bin;
   GtkWidget          *summary_inscription;
   GtkWidget          *vertical_box;
   GtkEventController *drag_source;
@@ -80,8 +80,6 @@ enum
 {
   PROP_0,
   PROP_CONTEXT,
-  PROP_DATE_END,
-  PROP_DATE_START,
   PROP_EVENT,
   PROP_TIMESTAMP_POLICY,
   PROP_ORIENTATION,
@@ -115,12 +113,19 @@ G_DEFINE_TYPE_WITH_CODE (GcalEventWidget, gcal_event_widget, GTK_TYPE_WIDGET,
  * Auxiliary methods
  */
 
-static gboolean
-get_vertical_enabled (gpointer        user_data,
-                      GcalEvent      *event,
-                      GtkOrientation  orientation)
+static void
+gcal_event_widget_update_orientation_widgets (GcalEventWidget *self)
 {
-  return orientation == GTK_ORIENTATION_VERTICAL && !gcal_event_get_all_day (event);
+  gboolean is_vertical;
+  gboolean is_all_day;
+
+  is_vertical = self->orientation == GTK_ORIENTATION_VERTICAL;
+  is_all_day = gcal_event_get_all_day (self->event);
+
+  if (is_vertical && !is_all_day)
+    gcal_overflow_bin_set_child (GCAL_OVERFLOW_BIN (self->overflow_bin), self->vertical_box);
+  else
+    gcal_overflow_bin_set_child (GCAL_OVERFLOW_BIN (self->overflow_bin), self->horizontal_box);
 }
 
 static void
@@ -195,9 +200,9 @@ update_color (GcalEventWidget *self)
 
   /* Fades out an event that's earlier than the current date */
   if (date_compare < 0)
-    gtk_widget_add_css_class (GTK_WIDGET (self), "dim-label");
+    gtk_widget_add_css_class (GTK_WIDGET (self), "dimmed");
   else
-    gtk_widget_remove_css_class (GTK_WIDGET (self), "dim-label");
+    gtk_widget_remove_css_class (GTK_WIDGET (self), "dimmed");
 
   /* Remove the old style class */
   if (self->css_class)
@@ -364,7 +369,7 @@ gcal_event_widget_set_event_tooltip (GcalEventWidget *self,
   if (location && location[0] != '\0')
     {
       g_autofree gchar *escaped_location = NULL;
-      g_autoptr (GUri) guri = NULL;
+      g_autoptr (GUri) G_GNUC_UNUSED guri = NULL; /* we just check for a parse error; don't care about the URI */
       g_autoptr (GString) string = NULL;
       g_autoptr (GError) error = NULL;
 
@@ -505,6 +510,13 @@ gcal_event_widget_set_event_internal (GcalEventWidget *self,
                            self,
                            G_CONNECT_SWAPPED);
 
+  g_signal_connect_object (self->context,
+                           "notify::time-format",
+                           G_CALLBACK (gcal_event_widget_update_timestamp),
+                           self,
+                           G_CONNECT_SWAPPED);
+
+
   /* Tooltip */
   gcal_event_widget_set_event_tooltip (self, event);
 
@@ -541,6 +553,9 @@ reply_preview_callback (GtkWidget              *event_popover,
 
   gtk_popover_popdown (GTK_POPOVER (event_popover));
   gtk_widget_unparent (event_popover);
+  gtk_accessible_update_state (GTK_ACCESSIBLE (data->event_widget),
+                               GTK_ACCESSIBLE_STATE_PRESSED, FALSE,
+                               -1);
 
   g_clear_pointer (&data, g_free);
 }
@@ -649,14 +664,6 @@ gcal_event_widget_set_property (GObject      *object,
                                G_CONNECT_SWAPPED);
       break;
 
-    case PROP_DATE_END:
-      gcal_event_widget_set_date_end (self, g_value_get_boxed (value));
-      break;
-
-    case PROP_DATE_START:
-      gcal_event_widget_set_date_start (self, g_value_get_boxed (value));
-      break;
-
     case PROP_EVENT:
       gcal_event_widget_set_event_internal (self, g_value_get_object (value));
       break;
@@ -667,7 +674,7 @@ gcal_event_widget_set_property (GObject      *object,
 
     case PROP_ORIENTATION:
       self->orientation = g_value_get_enum (value);
-      gtk_widget_set_visible (self->vertical_box, self->orientation == GTK_ORIENTATION_VERTICAL);
+      gcal_event_widget_update_orientation_widgets (self);
       gcal_event_widget_update_style (self);
       gcal_event_widget_update_timestamp (self);
       g_object_notify (object, "orientation");
@@ -692,14 +699,6 @@ gcal_event_widget_get_property (GObject      *object,
       g_value_set_object (value, self->context);
       break;
 
-    case PROP_DATE_END:
-      g_value_set_boxed (value, self->dt_end);
-      break;
-
-    case PROP_DATE_START:
-      g_value_set_boxed (value, self->dt_start);
-      break;
-
     case PROP_EVENT:
       g_value_set_object (value, self->event);
       break;
@@ -723,7 +722,8 @@ gcal_event_widget_dispose (GObject *object)
   GcalEventWidget *self = GCAL_EVENT_WIDGET (object);
 
   g_clear_pointer (&self->preview_popover, gtk_widget_unparent);
-  g_clear_pointer (&self->main_widget, gtk_widget_unparent);
+  g_clear_pointer (&self->overflow_bin, gtk_widget_unparent);
+  g_clear_pointer (&self->edge, gtk_widget_unparent);
 
   G_OBJECT_CLASS (gcal_event_widget_parent_class)->dispose (object);
 }
@@ -770,37 +770,6 @@ gcal_event_widget_class_init (GcalEventWidgetClass *klass)
                                                         "Context",
                                                         GCAL_TYPE_CONTEXT,
                                                         G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS));
-  /**
-   * GcalEventWidget::date-end:
-   *
-   * The end date this widget represents. Notice that this may
-   * differ from the event's end date. For example, if the event
-   * spans more than one month and we're in Month View, the end
-   * date marks the last day this event widget is visible.
-   */
-  g_object_class_install_property (object_class,
-                                   PROP_DATE_END,
-                                   g_param_spec_boxed ("date-end",
-                                                       "End date",
-                                                       "The end date of the widget",
-                                                       G_TYPE_DATE_TIME,
-                                                       G_PARAM_READWRITE));
-
-  /**
-   * GcalEventWidget::date-start:
-   *
-   * The start date this widget represents. Notice that this may
-   * differ from the event's start date. For example, if the event
-   * spans more than one month and we're in Month View, the start
-   * date marks the first day this event widget is visible.
-   */
-  g_object_class_install_property (object_class,
-                                   PROP_DATE_START,
-                                   g_param_spec_boxed ("date-start",
-                                                       "Start date",
-                                                       "The start date of the widget",
-                                                       G_TYPE_DATE_TIME,
-                                                       G_PARAM_READWRITE));
 
   /**
    * GcalEventWidget::event:
@@ -841,32 +810,57 @@ gcal_event_widget_class_init (GcalEventWidgetClass *klass)
 
   signals[ACTIVATE] = g_signal_new ("activate",
                                      GCAL_TYPE_EVENT_WIDGET,
-                                     G_SIGNAL_RUN_LAST,
+                                     G_SIGNAL_RUN_FIRST | G_SIGNAL_ACTION,
                                      0,
                                      NULL, NULL,
                                      g_cclosure_marshal_VOID__VOID,
                                      G_TYPE_NONE,
                                      0);
 
+  gtk_widget_class_set_activate_signal (widget_class, signals[ACTIVATE]);
+
+  {
+    g_autoptr (GtkShortcutAction) activate_action = NULL;
+    const guint activate_keyvals[] = {
+      GDK_KEY_space,
+      GDK_KEY_KP_Space,
+      GDK_KEY_Return,
+      GDK_KEY_ISO_Enter,
+      GDK_KEY_KP_Enter,
+    };
+
+    activate_action = gtk_signal_action_new ("activate");
+
+    for (size_t i = 0; i < G_N_ELEMENTS (activate_keyvals); i++)
+      {
+        g_autoptr (GtkShortcut) activate_shortcut = NULL;
+
+        activate_shortcut = gtk_shortcut_new (gtk_keyval_trigger_new (activate_keyvals[i], 0),
+                                              g_object_ref (activate_action));
+
+        gtk_widget_class_add_shortcut (widget_class, activate_shortcut);
+      }
+  }
+
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/calendar/ui/gui/gcal-event-widget.ui");
 
   gtk_widget_class_bind_template_child (widget_class, GcalEventWidget, drag_source);
   gtk_widget_class_bind_template_child (widget_class, GcalEventWidget, horizontal_box);
-  gtk_widget_class_bind_template_child (widget_class, GcalEventWidget, main_widget);
+  gtk_widget_class_bind_template_child (widget_class, GcalEventWidget, edge);
+  gtk_widget_class_bind_template_child (widget_class, GcalEventWidget, overflow_bin);
   gtk_widget_class_bind_template_child (widget_class, GcalEventWidget, timestamp_label);
-  gtk_widget_class_bind_template_child (widget_class, GcalEventWidget, squeezer);
   gtk_widget_class_bind_template_child (widget_class, GcalEventWidget, summary_inscription);
   gtk_widget_class_bind_template_child (widget_class, GcalEventWidget, vertical_box);
 
-  gtk_widget_class_bind_template_callback (widget_class, get_vertical_enabled);
   gtk_widget_class_bind_template_callback (widget_class, on_click_gesture_pressed_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_click_gesture_release_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_drag_source_begin_cb);
   gtk_widget_class_bind_template_callback (widget_class, on_drag_source_prepare_cb);
 
   gtk_widget_class_set_css_name (widget_class, "event");
+  gtk_widget_class_set_accessible_role (widget_class, GTK_ACCESSIBLE_ROLE_TOGGLE_BUTTON);
 
-  gtk_widget_class_set_layout_manager_type (widget_class, GTK_TYPE_BIN_LAYOUT);
+  gtk_widget_class_set_layout_manager_type (widget_class, GTK_TYPE_BOX_LAYOUT);
 }
 
 static void
@@ -881,6 +875,10 @@ gcal_event_widget_init (GcalEventWidget *self)
   /* Starts with horizontal */
   self->orientation = GTK_ORIENTATION_HORIZONTAL;
   gtk_widget_add_css_class (GTK_WIDGET (self), "horizontal");
+
+  gtk_accessible_update_state (GTK_ACCESSIBLE (self),
+                               GTK_ACCESSIBLE_STATE_PRESSED, FALSE,
+                               -1);
 }
 
 GtkWidget*
@@ -918,6 +916,11 @@ gcal_event_widget_get_date_end (GcalEventWidget *self)
  * Sets the visible end date of this widget. This
  * may differ from the event's end date, but cannot
  * be after it.
+ *
+ * The end date for this widget may differ from the event's end
+ * date. For example, if the event spans more than one month and we're
+ * in Month View, the end date marks the last day this event widget is
+ * visible.
  */
 void
 gcal_event_widget_set_date_end (GcalEventWidget *self,
@@ -937,8 +940,6 @@ gcal_event_widget_set_date_end (GcalEventWidget *self,
       self->dt_end = g_date_time_ref (date_end);
 
       gcal_event_widget_update_style (self);
-
-      g_object_notify (G_OBJECT (self), "date-end");
     }
 }
 
@@ -962,11 +963,16 @@ gcal_event_widget_get_date_start (GcalEventWidget *self)
 /**
  * gcal_event_widget_set_date_start:
  * @self: a #GcalEventWidget
- * @date_end: the start date of this widget
+ * @date_start: the start date of this widget
  *
  * Sets the visible start date of this widget. This
  * may differ from the event's start date, but cannot
  * be before it.
+ *
+ * The start date for this widget may differ from the event's start
+ * date. For example, if the event spans more than one month and we're
+ * in Month View, the start date marks the first day this event widget
+ * is visible.
  */
 void
 gcal_event_widget_set_date_start (GcalEventWidget *self,
@@ -986,8 +992,6 @@ gcal_event_widget_set_date_start (GcalEventWidget *self,
       self->dt_start = g_date_time_ref (date_start);
 
       gcal_event_widget_update_style (self);
-
-      g_object_notify (G_OBJECT (self), "date-start");
     }
 }
 
@@ -1049,6 +1053,9 @@ gcal_event_widget_show_preview (GcalEventWidget          *self,
   g_signal_connect (self->preview_popover, "closed", G_CALLBACK (on_event_popover_closed_cb), data);
   g_signal_connect (self->preview_popover, "edit", G_CALLBACK (on_event_popover_edit_cb), data);
   gtk_popover_popup (GTK_POPOVER (self->preview_popover));
+  gtk_accessible_update_state (GTK_ACCESSIBLE (self),
+                               GTK_ACCESSIBLE_STATE_PRESSED, TRUE,
+                               -1);
 
   GCAL_EXIT;
 }
