@@ -26,6 +26,7 @@
 #include "gcal-clock.h"
 #include "gcal-debug.h"
 #include "gcal-event-widget.h"
+#include "gcal-gui-utils.h"
 #include "gcal-utils.h"
 #include "gcal-view-private.h"
 #include "gcal-week-header.h"
@@ -69,18 +70,13 @@ typedef struct
 
 struct _GcalWeekHeader
 {
-  GtkBox              parent;
+  GtkWidget           parent;
 
   GtkGrid            *grid;
-  GtkWidget          *month_label;
-  GtkWidget          *week_label;
-  GtkWidget          *year_label;
   GtkWidget          *scrolledwindow;
-  GtkButton          *expand_button;
-  GtkWidget          *expand_button_box;
-  GtkWidget          *header_labels_box;
   GtkEventController *motion_controller;
   GtkBox             *weekdays_box;
+  GtkWidget          *main_box;
 
   GcalContext        *context;
 
@@ -99,18 +95,24 @@ struct _GcalWeekHeader
    * Used for checking if the header is in collapsed state or expand state
    * false is collapse state true is expand state
    */
+  gboolean            can_expand;
   gboolean            expanded;
 
   GDateTime          *active_date;
 
-  gint                selection_start;
-  gint                selection_end;
-  gint                dnd_cell;
+  struct {
+    gint              start;
+    gint              end;
+    GtkWidget        *widget;
+  } selection;
+
+  struct {
+    gint              cell;
+    GtkWidget        *widget;
+  } dnd;
 
   /* Array of nullable weather infos for each day, starting with Sunday. */
   WeatherInfoDay      weather_infos[7];
-
-  GtkSizeGroup       *sizegroup;
 };
 
 typedef enum
@@ -119,6 +121,14 @@ typedef enum
   DOWN
 } MoveDirection;
 
+enum
+{
+  PROP_0,
+  PROP_CAN_EXPAND,
+  PROP_EXPANDED,
+  N_PROPS,
+};
+
 enum
 {
   EVENT_ACTIVATED,
@@ -126,8 +136,9 @@ enum
 };
 
 static guint signals[LAST_SIGNAL] = { 0, };
+static GParamSpec *properties [N_PROPS];
 
-G_DEFINE_TYPE (GcalWeekHeader, gcal_week_header, GTK_TYPE_BOX);
+G_DEFINE_TYPE (GcalWeekHeader, gcal_week_header, GTK_TYPE_WIDGET);
 
 
 /* WeatherInfoDay methods */
@@ -261,10 +272,10 @@ on_button_pressed (GtkGestureClick *click_gesture,
   column_width = width / 7.0;
   column = ltr ? (x / column_width) : (7  - x / column_width);
 
-  self->selection_start = column;
-  self->selection_end = column;
+  self->selection.start = column;
+  self->selection.end = column;
 
-  gtk_widget_queue_draw (GTK_WIDGET (self));
+  gtk_widget_set_visible (self->selection.widget, TRUE);
 
   gtk_event_controller_set_propagation_phase (self->motion_controller,
                                               GTK_PHASE_BUBBLE);
@@ -286,9 +297,9 @@ on_motion_notify (GtkEventControllerMotion *motion_event,
   column_width = width / 7.0;
   column = ltr ? (x / column_width) : (7  - x / column_width);
 
-  self->selection_end = column;
+  self->selection.end = column;
 
-  gtk_widget_queue_draw (GTK_WIDGET (self));
+  gtk_widget_queue_allocate (GTK_WIDGET (self));
 }
 
 static void
@@ -302,10 +313,11 @@ on_button_released (GtkGestureClick *click_gesture,
   g_autoptr (GDateTime) selection_end = NULL;
   g_autoptr (GDateTime) week_start = NULL;
   g_autoptr (GcalRange) range = NULL;
+  graphene_point_t out;
   GtkWidget *weekview;
   gboolean ltr;
   gdouble column_width;
-  gdouble out_x, out_y;
+  gdouble local_x;
   gint column;
   gint width;
   gint start;
@@ -316,15 +328,14 @@ on_button_released (GtkGestureClick *click_gesture,
   column_width = width / 7.0;
   column = ltr ? (x / column_width) : (7  - x / column_width);
 
-  self->selection_end = column;
-
-  gtk_widget_queue_draw (GTK_WIDGET (self));
+  self->selection.end = column;
+  gtk_widget_queue_allocate (GTK_WIDGET (self));
 
   /* Fake the week view's event so we can control the X and Y values */
   weekview = gtk_widget_get_ancestor (GTK_WIDGET (self), GCAL_TYPE_WEEK_VIEW);
 
-  start = self->selection_start;
-  end = self->selection_end;
+  start = self->selection.start;
+  end = self->selection.end;
 
   if (start > end)
     {
@@ -337,21 +348,16 @@ on_button_released (GtkGestureClick *click_gesture,
   selection_start = g_date_time_add_days (week_start, start);
   selection_end = end == start ? g_date_time_ref (selection_start) : g_date_time_add_days (week_start, end + 1);
 
-  out_x = ltr ? (column_width * (column + 0.5)) : (width - column_width * (column + 0.5));
+  local_x = ltr ? (column_width * (column + 0.5)) : (width - column_width * (column + 0.5));
 
-  /* Translate X... */
-  gtk_widget_translate_coordinates (self->scrolledwindow, weekview, out_x, 0, &out_x, NULL);
-
-  /* And Y */
-  gtk_widget_translate_coordinates (GTK_WIDGET (self),
-                                    weekview,
-                                    0,
-                                    gtk_widget_get_height (GTK_WIDGET (self)),
-                                    NULL,
-                                    &out_y);
+  if (!gtk_widget_compute_point (GTK_WIDGET (self),
+                                 weekview,
+                                 &GRAPHENE_POINT_INIT (local_x, gtk_widget_get_height (GTK_WIDGET (self))),
+                                 &out))
+    g_assert_not_reached ();
 
   range = gcal_range_new (selection_start, selection_end, GCAL_RANGE_DATE_ONLY);
-  gcal_view_create_event (GCAL_VIEW (weekview), range, out_x, out_y);
+  gcal_view_create_event (GCAL_VIEW (weekview), range, out.x, out.y);
 
   gtk_event_controller_set_propagation_phase (self->motion_controller, GTK_PHASE_NONE);
 }
@@ -498,7 +504,11 @@ update_overflow (GcalWeekHeader *self)
         }
     }
 
-  gtk_widget_set_visible (GTK_WIDGET (self->expand_button), show_expand);
+  if (self->can_expand != show_expand)
+    {
+      self->can_expand = show_expand;
+      g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_CAN_EXPAND]);
+    }
 }
 
 static void
@@ -582,7 +592,6 @@ check_mergeable_events (GcalWeekHeader *self)
             }
 
           /* We found events to merge. Lets merge them */
-          i = 0;
 
           for (i = 0; i < events_to_merge; i++)
             {
@@ -969,49 +978,16 @@ update_unchanged_events (GcalWeekHeader *self,
 static void
 update_title (GcalWeekHeader *self)
 {
-  GDateTime *week_start, *week_end, *week_mid;
-  gchar *year_label, *month_label, *week_label;
+  g_autoptr (GDateTime) week_start = NULL;
   gint today_column;
-  gint i;
 
   if(!self->active_date)
     return;
 
   week_start = gcal_date_time_get_start_of_week (self->active_date);
-  week_end = g_date_time_add_days (week_start, 6);
-  week_mid = g_date_time_add_days (week_start, 3);
-
-  if (g_date_time_get_month (week_start) == g_date_time_get_month (week_end))
-    {
-      month_label = g_strdup_printf ("%s", gcal_get_month_name (g_date_time_get_month (week_start) - 1));
-    }
-  else
-    {
-      month_label = g_strdup_printf("%s–%s ",
-                                    gcal_get_month_name (g_date_time_get_month (week_start) -1),
-                                    gcal_get_month_name (g_date_time_get_month (week_end) - 1));
-    }
-
-  if (g_date_time_get_year (week_start) == g_date_time_get_year (week_end))
-    {
-      year_label = g_strdup_printf ("%d", g_date_time_get_year (week_start));
-    }
-  else
-    {
-      year_label = g_strdup_printf ("%d–%d",
-                                    g_date_time_get_year (week_start),
-                                    g_date_time_get_year (week_end));
-    }
-
-  week_label = g_strdup_printf (_("Week %d"), g_date_time_get_week_of_year (week_mid));
-
-  gtk_label_set_label (GTK_LABEL (self->month_label), month_label);
-  gtk_label_set_label (GTK_LABEL (self->week_label), week_label);
-  gtk_label_set_label (GTK_LABEL (self->year_label), year_label);
-
   today_column = get_today_column (self);
 
-  for (i = 0; i < 7; i++)
+  for (gint i = 0; i < 7; i++)
     {
       g_autoptr (GDateTime) day = NULL;
       g_autofree gchar *weekday_date = NULL;
@@ -1032,15 +1008,15 @@ update_title (GcalWeekHeader *self)
         {
           gtk_widget_add_css_class (header->weekday_name_label, "accent");
           gtk_widget_add_css_class (header->day_number_label, "accent");
-          gtk_widget_remove_css_class (header->weekday_name_label, "dim-label");
-          gtk_widget_remove_css_class (header->day_number_label, "dim-label");
+          gtk_widget_remove_css_class (header->weekday_name_label, "dimmed");
+          gtk_widget_remove_css_class (header->day_number_label, "dimmed");
         }
       else
         {
           gtk_widget_remove_css_class (header->weekday_name_label, "accent");
           gtk_widget_remove_css_class (header->day_number_label, "accent");
-          gtk_widget_add_css_class (header->weekday_name_label, "dim-label");
-          gtk_widget_add_css_class (header->day_number_label, "dim-label");
+          gtk_widget_add_css_class (header->weekday_name_label, "dimmed");
+          gtk_widget_add_css_class (header->day_number_label, "dimmed");
         }
 
       weekday_date = g_strdup_printf ("%d", n_day);
@@ -1050,13 +1026,6 @@ update_title (GcalWeekHeader *self)
       weekday_abv = g_utf8_strup (weekday, -1);
       gtk_label_set_label (GTK_LABEL (header->weekday_name_label), weekday_abv);
     }
-
-  g_clear_pointer (&week_start, g_date_time_unref);
-  g_clear_pointer (&week_end, g_date_time_unref);
-  g_clear_pointer (&week_mid, g_date_time_unref);
-  g_clear_pointer (&month_label, g_free);
-  g_clear_pointer (&week_label, g_free);
-  g_clear_pointer (&year_label, g_free);
 }
 
 static void
@@ -1071,7 +1040,6 @@ header_collapse (GcalWeekHeader *self)
                                   GTK_POLICY_NEVER,
                                   GTK_POLICY_NEVER);
   gtk_scrolled_window_set_max_content_height (GTK_SCROLLED_WINDOW (self->scrolledwindow), -1);
-  gtk_button_set_icon_name (self->expand_button, "go-down-symbolic");
 
   layout_manager = gtk_widget_get_layout_manager (GTK_WIDGET (self->grid));
 
@@ -1114,8 +1082,6 @@ header_expand (GcalWeekHeader *self)
   gtk_scrolled_window_set_max_content_height (GTK_SCROLLED_WINDOW (self->scrolledwindow),
                                               gtk_widget_get_height (week_view) / 2);
 
-  gtk_button_set_icon_name (self->expand_button, "go-up-symbolic");
-
   child = gtk_widget_get_first_child (GTK_WIDGET (self->grid));
   while (child)
     {
@@ -1289,14 +1255,29 @@ on_drop_target_drop_cb (GtkDropTarget  *drop_target,
   GCAL_RETURN (TRUE);
 }
 
+static GdkDragAction
+on_drop_target_enter_cb (GtkDropTarget  *drop_target,
+                         gdouble         x,
+                         gdouble         y,
+                         GcalWeekHeader *self)
+{
+
+  GCAL_ENTRY;
+
+  self->dnd.cell = get_dnd_cell (self, x, y);
+  gtk_widget_set_visible (self->dnd.widget, self->dnd.cell != -1);
+
+  GCAL_RETURN (self->dnd.cell != -1 ? GDK_ACTION_COPY : 0);
+}
+
 static void
 on_drop_target_leave_cb (GtkDropTarget  *drop_target,
                          GcalWeekHeader *self)
 {
   GCAL_ENTRY;
 
-  self->dnd_cell = -1;
-  gtk_widget_queue_draw (GTK_WIDGET (self));
+  self->dnd.cell = -1;
+  gtk_widget_set_visible (self->dnd.widget, FALSE);
 
   GCAL_EXIT;
 }
@@ -1309,81 +1290,64 @@ on_drop_target_motion_cb (GtkDropTarget  *drop_target,
 {
   GCAL_ENTRY;
 
-  self->dnd_cell = get_dnd_cell (self, x, y);
-  gtk_widget_queue_draw (GTK_WIDGET (self));
+  self->dnd.cell = get_dnd_cell (self, x, y);
+  gtk_widget_set_visible (self->dnd.widget, self->dnd.cell != -1);
+  gtk_widget_queue_allocate (GTK_WIDGET (self));
 
-  GCAL_RETURN (self->dnd_cell != -1 ? GDK_ACTION_COPY : 0);
+  GCAL_RETURN (self->dnd.cell != -1 ? GDK_ACTION_COPY : 0);
 }
 
 /* Drawing area content and size */
 
 static void
-gcal_week_header_snapshot (GtkWidget   *widget,
-                           GtkSnapshot *snapshot)
+gcal_week_header_measure (GtkWidget      *widget,
+                          GtkOrientation  orientation,
+                          gint            for_size,
+                          gint           *minimum,
+                          gint           *natural,
+                          gint           *minimum_baseline,
+                          gint           *natural_baseline)
 {
-  GtkStyleContext *context;
-  GcalWeekHeader *self;
-  GtkStateFlags state;
-  int alloc_width;
-  int alloc_height;
-  GDateTime *week_start, *week_end;
+  GcalWeekHeader *self = (GcalWeekHeader *) widget;
+
+  g_assert (GCAL_IS_WEEK_HEADER (self));
+
+  gtk_widget_measure (self->main_box,
+                      orientation,
+                      for_size,
+                      minimum, natural,
+                      minimum_baseline,
+                      natural_baseline);
+}
+
+static void
+gcal_week_header_size_allocate (GtkWidget *widget,
+                                gint       width,
+                                gint       height,
+                                gint       baseline)
+{
+  GcalWeekHeader *self = (GcalWeekHeader *) widget;
   gboolean ltr;
   gdouble cell_width;
-  gint current_cell;
-  gint start_x;
-  gint start_y;
-  gint height;
-  gint width;
-  gint x;
-  gint y;
 
-  /* Fonts and colour selection */
-  self = GCAL_WEEK_HEADER (widget);
-  context = gtk_widget_get_style_context (widget);
-  state = gtk_style_context_get_state (context);
+  g_assert (GCAL_IS_WEEK_HEADER (self));
+
+  gtk_widget_allocate (self->main_box, width, height, baseline, NULL);
+
   ltr = gtk_widget_get_direction (widget) != GTK_TEXT_DIR_RTL;
+  cell_width = width / 7.0;
 
-  start_x = ltr ? gtk_widget_get_width (self->expand_button_box) : 0;
-  start_y = gtk_widget_get_height (self->header_labels_box);
-
-  alloc_width = gtk_widget_get_width (widget);
-  alloc_height = gtk_widget_get_height (widget);
-
-  if (!ltr)
-    alloc_width -= gtk_widget_get_width (self->expand_button_box);
-
-  week_start = gcal_date_time_get_start_of_week (self->active_date);
-  week_end = g_date_time_add_days (week_start, 6);
-  current_cell = g_date_time_get_day_of_week (self->active_date) - 1;
-  current_cell = (7 + current_cell - self->first_weekday) % 7;
-
-  cell_width = (alloc_width - start_x) / 7.0;
-
-  /* Drag and Drop highlight */
-  if (self->dnd_cell != -1)
-    {
-      gtk_style_context_save (context);
-      gtk_style_context_add_class (context, "dnd");
-
-      gtk_snapshot_render_background (snapshot,
-                                      context,
-                                      start_x + self->dnd_cell * cell_width,
-                                      start_y,
-                                      cell_width,
-                                      alloc_height - start_y);
-
-      gtk_style_context_restore (context);
-    }
-
-  /* Draw the selection background */
-  if (self->selection_start != -1 && self->selection_end != -1)
+  if (gtk_widget_should_layout (self->selection.widget))
     {
       gint selection_width;
       gint selection_x;
       gint start, end;
 
-      start = self->selection_start;
-      end = self->selection_end;
+      g_assert (self->selection.start != -1);
+      g_assert (self->selection.end != -1);
+
+      start = self->selection.start;
+      end = self->selection.end;
 
       if (start > end)
         {
@@ -1392,43 +1356,59 @@ gcal_week_header_snapshot (GtkWidget   *widget,
           start = start - end;
         }
 
-      gtk_style_context_save (context);
-      gtk_style_context_set_state (context, state | GTK_STATE_FLAG_SELECTED);
-
       selection_width = (end - start + 1) * cell_width;
-      selection_x = ltr ? (start * cell_width) : (alloc_width - (start * cell_width + selection_width));
+      selection_x = ltr ? (start * cell_width) : (width - (start * cell_width + selection_width));
 
-      gtk_snapshot_render_background (snapshot,
-                                      context,
-                                      ALIGNED (start_x + selection_x) + 0.33,
-                                      start_y - 6,
-                                      ALIGNED (selection_width + 1),
-                                      alloc_height - start_y + 6);
 
-      gtk_snapshot_render_frame (snapshot,
-                                 context,
-                                 ALIGNED (start_x + selection_x) + 0.33,
-                                 start_y - 6,
-                                 ALIGNED (selection_width + 1),
-                                 alloc_height - start_y + 6);
-
-      gtk_style_context_restore (context);
+      gtk_widget_size_allocate (self->selection.widget,
+                                &(GtkAllocation) {
+                                  .x = ALIGNED (selection_x),
+                                  .y = -6,
+                                  .width = ALIGNED (selection_width + 1),
+                                  .height = height + 6,
+                                },
+                                baseline);
     }
 
-  x = ALIGNED (ltr ? start_x : alloc_width - start_x);
-  y = start_y;
-  width = gtk_widget_get_width (widget) - start_x;
-  height = gtk_widget_get_height (widget) - start_y;
+  if (gtk_widget_should_layout (self->dnd.widget))
+    {
+      gtk_widget_size_allocate (self->dnd.widget,
+                                &(GtkAllocation) {
+                                  .x = self->dnd.cell * cell_width,
+                                  .y = -6,
+                                  .width = cell_width,
+                                  .height = height + 6,
+                                },
+                                baseline);
+    }
+}
+
+static void
+gcal_week_header_snapshot (GtkWidget   *widget,
+                           GtkSnapshot *snapshot)
+{
+  GcalWeekHeader *self;
+  gboolean ltr;
+  gint height;
+  gint width;
+  gint x;
+
+  /* Fonts and colour selection */
+  self = GCAL_WEEK_HEADER (widget);
+  ltr = gtk_widget_get_direction (widget) != GTK_TEXT_DIR_RTL;
+
+  width = gtk_widget_get_width (widget);
+  height = gtk_widget_get_height (widget);
+  x = ALIGNED (ltr ? 0 : width);
 
   gtk_snapshot_save (snapshot);
-  gtk_snapshot_translate (snapshot, &GRAPHENE_POINT_INIT (x, y));
+  gtk_snapshot_translate (snapshot, &GRAPHENE_POINT_INIT (x, 0));
   gcal_week_view_common_snapshot_hour_lines (widget, snapshot, GTK_ORIENTATION_HORIZONTAL, width, height);
   gtk_snapshot_restore (snapshot);
 
-  GTK_WIDGET_CLASS (gcal_week_header_parent_class)->snapshot (widget, snapshot);
-
-  g_clear_pointer (&week_start, g_date_time_unref);
-  g_clear_pointer (&week_end, g_date_time_unref);
+  gtk_widget_snapshot_child (widget, self->selection.widget, snapshot);
+  gtk_widget_snapshot_child (widget, self->dnd.widget, snapshot);
+  gtk_widget_snapshot_child (widget, self->main_box, snapshot);
 }
 
 
@@ -1436,6 +1416,21 @@ gcal_week_header_snapshot (GtkWidget   *widget,
  * GObject overrides
  */
 
+static void
+gcal_week_header_dispose (GObject *object)
+{
+  GcalWeekHeader *self = (GcalWeekHeader *) object;
+
+  g_assert (GCAL_IS_WEEK_HEADER (self));
+
+  g_clear_pointer (&self->dnd.widget, gtk_widget_unparent);
+  g_clear_pointer (&self->selection.widget, gtk_widget_unparent);
+
+  gtk_widget_dispose_template (GTK_WIDGET (self), GCAL_TYPE_WEEK_HEADER);
+
+  G_OBJECT_CLASS (gcal_week_header_parent_class)->dispose (object);
+}
+
 static void
 gcal_week_header_finalize (GObject *object)
 {
@@ -1453,16 +1448,74 @@ gcal_week_header_finalize (GObject *object)
   G_OBJECT_CLASS (gcal_week_header_parent_class)->finalize (object);
 }
 
+static void
+gcal_week_header_get_property (GObject    *object,
+                               guint       prop_id,
+                               GValue     *value,
+                               GParamSpec *pspec)
+{
+  GcalWeekHeader *self = GCAL_WEEK_HEADER (object);
+
+  switch (prop_id)
+    {
+    case PROP_CAN_EXPAND:
+      g_value_set_boolean (value, self->can_expand);
+      break;
+
+    case PROP_EXPANDED:
+      g_value_set_boolean (value, self->expanded);
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+    }
+}
+
+static void
+gcal_week_header_set_property (GObject      *object,
+                               guint         prop_id,
+                               const GValue *value,
+                               GParamSpec   *pspec)
+{
+  GcalWeekHeader *self = GCAL_WEEK_HEADER (object);
+
+  switch (prop_id)
+    {
+    case PROP_EXPANDED:
+      gcal_week_header_set_expanded (self, g_value_get_boolean (value));
+      break;
+
+    case PROP_CAN_EXPAND:
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+    }
+}
+
 static void
 gcal_week_header_class_init (GcalWeekHeaderClass *kclass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (kclass);
   GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (kclass);
 
+  object_class->dispose = gcal_week_header_dispose;
   object_class->finalize = gcal_week_header_finalize;
+  object_class->get_property = gcal_week_header_get_property;
+  object_class->set_property = gcal_week_header_set_property;
 
+  widget_class->measure = gcal_week_header_measure;
+  widget_class->size_allocate = gcal_week_header_size_allocate;
   widget_class->snapshot = gcal_week_header_snapshot;
 
+  properties[PROP_CAN_EXPAND] = g_param_spec_boolean ("can-expand", NULL, NULL,
+                                                      FALSE,
+                                                      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+
+  properties[PROP_EXPANDED] = g_param_spec_boolean ("expanded", NULL, NULL,
+                                                    FALSE,
+                                                    G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+
+  g_object_class_install_properties (object_class, N_PROPS, properties);
+
   signals[EVENT_ACTIVATED] = g_signal_new ("event-activated",
                                            GCAL_TYPE_WEEK_HEADER,
                                            G_SIGNAL_RUN_FIRST,
@@ -1473,24 +1526,18 @@ gcal_week_header_class_init (GcalWeekHeaderClass *kclass)
 
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gnome/calendar/ui/views/gcal-week-header.ui");
 
-  gtk_widget_class_bind_template_child (widget_class, GcalWeekHeader, expand_button);
-  gtk_widget_class_bind_template_child (widget_class, GcalWeekHeader, expand_button_box);
   gtk_widget_class_bind_template_child (widget_class, GcalWeekHeader, grid);
-  gtk_widget_class_bind_template_child (widget_class, GcalWeekHeader, header_labels_box);
-  gtk_widget_class_bind_template_child (widget_class, GcalWeekHeader, month_label);
+  gtk_widget_class_bind_template_child (widget_class, GcalWeekHeader, main_box);
   gtk_widget_class_bind_template_child (widget_class, GcalWeekHeader, motion_controller);
   gtk_widget_class_bind_template_child (widget_class, GcalWeekHeader, scrolledwindow);
-  gtk_widget_class_bind_template_child (widget_class, GcalWeekHeader, sizegroup);
-  gtk_widget_class_bind_template_child (widget_class, GcalWeekHeader, week_label);
   gtk_widget_class_bind_template_child (widget_class, GcalWeekHeader, weekdays_box);
-  gtk_widget_class_bind_template_child (widget_class, GcalWeekHeader, year_label);
 
   gtk_widget_class_bind_template_callback (widget_class, on_button_pressed);
   gtk_widget_class_bind_template_callback (widget_class, on_button_released);
   gtk_widget_class_bind_template_callback (widget_class, on_expand_action_activated);
   gtk_widget_class_bind_template_callback (widget_class, on_motion_notify);
 
-  gtk_widget_class_set_css_name (widget_class, "calendar-view");
+  gtk_widget_class_set_css_name (widget_class, "weekheader");
 }
 
 static void
@@ -1500,9 +1547,18 @@ gcal_week_header_init (GcalWeekHeader *self)
   gint i;
 
   self->expanded = FALSE;
-  self->selection_start = -1;
-  self->selection_end = -1;
-  self->dnd_cell = -1;
+
+  self->selection.start = -1;
+  self->selection.end = -1;
+  self->selection.widget = gcal_create_selection_widget ();
+  gtk_widget_set_visible (self->selection.widget, FALSE);
+  gtk_widget_set_parent (self->selection.widget, GTK_WIDGET (self));
+
+  self->dnd.cell = -1;
+  self->dnd.widget = gcal_create_drop_target_widget ();
+  gtk_widget_set_visible (self->dnd.widget, FALSE);
+  gtk_widget_set_parent (self->dnd.widget, GTK_WIDGET (self));
+
   self->first_weekday = get_first_weekday ();
 
   gtk_widget_init_template (GTK_WIDGET (self));
@@ -1518,14 +1574,14 @@ gcal_week_header_init (GcalWeekHeader *self)
       header->weekday_name_label = gtk_label_new ("");
       gtk_widget_set_hexpand (header->weekday_name_label, TRUE);
       gtk_widget_add_css_class (header->weekday_name_label, "heading");
-      gtk_widget_add_css_class (header->weekday_name_label, "dim-label");
+      gtk_widget_add_css_class (header->weekday_name_label, "dimmed");
       gtk_label_set_xalign (GTK_LABEL (header->weekday_name_label), 0.0);
       gtk_box_append (GTK_BOX (box), header->weekday_name_label);
 
       header->day_number_label = gtk_label_new ("");
       gtk_widget_set_hexpand (header->day_number_label, TRUE);
       gtk_widget_add_css_class (header->day_number_label, "title-2");
-      gtk_widget_add_css_class (header->day_number_label, "dim-label");
+      gtk_widget_add_css_class (header->day_number_label, "dimmed");
       gtk_label_set_xalign (GTK_LABEL (header->day_number_label), 0.0);
       gtk_box_append (GTK_BOX (box), header->day_number_label);
 
@@ -1537,6 +1593,7 @@ gcal_week_header_init (GcalWeekHeader *self)
 
   drop_target = gtk_drop_target_new (GCAL_TYPE_EVENT_WIDGET, GDK_ACTION_COPY);
   g_signal_connect (drop_target, "drop", G_CALLBACK (on_drop_target_drop_cb), self);
+  g_signal_connect (drop_target, "enter", G_CALLBACK (on_drop_target_enter_cb), self);
   g_signal_connect (drop_target, "leave", G_CALLBACK (on_drop_target_leave_cb), self);
   g_signal_connect (drop_target, "motion", G_CALLBACK (on_drop_target_motion_cb), self);
   gtk_widget_add_controller (GTK_WIDGET (self), GTK_EVENT_CONTROLLER (drop_target));
@@ -1723,23 +1780,14 @@ gcal_week_header_get_children_by_uuid (GcalWeekHeader        *self,
   GCAL_RETURN (g_steal_pointer (&result));
 }
 
-GtkSizeGroup*
-gcal_week_header_get_sidebar_size_group (GcalWeekHeader *self)
-{
-  g_return_val_if_fail (GCAL_IS_WEEK_HEADER (self), NULL);
-
-  return self->sizegroup;
-}
-
 void
 gcal_week_header_clear_marks (GcalWeekHeader *self)
 {
   g_return_if_fail (GCAL_IS_WEEK_HEADER (self));
 
-  self->selection_start = -1;
-  self->selection_end = -1;
-
-  gtk_widget_queue_draw (GTK_WIDGET (self));
+  self->selection.start = -1;
+  self->selection.end = -1;
+  gtk_widget_set_visible (self->selection.widget, FALSE);
 }
 
 void
@@ -1772,3 +1820,35 @@ gcal_week_header_set_date (GcalWeekHeader *self,
   update_weather_infos (self);
 }
 
+gboolean
+gcal_week_header_get_can_expand (GcalWeekHeader *self)
+{
+  g_assert (GCAL_IS_WEEK_HEADER (self));
+
+  return self->can_expand;
+}
+
+gboolean
+gcal_week_header_get_expanded (GcalWeekHeader *self)
+{
+  g_assert (GCAL_IS_WEEK_HEADER (self));
+
+  return self->expanded;
+}
+
+void
+gcal_week_header_set_expanded (GcalWeekHeader *self,
+                               gboolean        expanded)
+{
+  g_assert (GCAL_IS_WEEK_HEADER (self));
+
+  if (self->expanded == expanded)
+    return;
+
+  if (expanded)
+    header_expand (self);
+  else
+    header_collapse (self);
+
+  g_object_notify_by_pspec (G_OBJECT (self), properties[PROP_EXPANDED]);
+}
