@@ -30,7 +30,7 @@ G_BEGIN_DECLS
 
 #define GCAL_TYPE_WEEK_HEADER (gcal_week_header_get_type())
 
-G_DECLARE_FINAL_TYPE (GcalWeekHeader, gcal_week_header, GCAL, WEEK_HEADER, GtkBox)
+G_DECLARE_FINAL_TYPE (GcalWeekHeader, gcal_week_header, GCAL, WEEK_HEADER, GtkWidget)
 
 void                 gcal_week_header_set_context                (GcalWeekHeader     *self,
                                                                   GcalContext        *context);
@@ -45,13 +45,17 @@ GList*               gcal_week_header_get_children_by_uuid       (GcalWeekHeader
                                                                   GcalRecurrenceModType  mod,
                                                                   const gchar           *uuid);
 
-GtkSizeGroup*        gcal_week_header_get_sidebar_size_group     (GcalWeekHeader     *self);
-
 void                 gcal_week_header_clear_marks                (GcalWeekHeader     *self);
 
 void                 gcal_week_header_set_date                   (GcalWeekHeader     *self,
                                                                   GDateTime          *date);
 
+gboolean             gcal_week_header_get_can_expand             (GcalWeekHeader     *self);
+
+gboolean             gcal_week_header_get_expanded               (GcalWeekHeader     *self);
+void                 gcal_week_header_set_expanded               (GcalWeekHeader     *self,
+                                                                  gboolean            expanded);
+
 G_END_DECLS
 
 #endif /* GCAL_WEEK_HEADER_H */
