@@ -36,6 +36,11 @@ GTimeZone*           gcal_weather_service_get_time_zone             (GcalWeather
 void                 gcal_weather_service_set_time_zone             (GcalWeatherService *self,
                                                                      GTimeZone          *value);
 
+GWeatherLocation*   gcal_weather_service_get_location               (GcalWeatherService *self);
+
+void                 gcal_weather_service_set_location              (GcalWeatherService *self,
+                                                                     GWeatherLocation   *value);
+
 GcalWeatherInfo*     gcal_weather_service_get_weather_info_for_date (GcalWeatherService *self,
                                                                      GDate              *date);
 
@@ -45,11 +50,18 @@ const gchar*         gcal_weather_service_get_attribution           (GcalWeather
 
 void                 gcal_weather_service_update                    (GcalWeatherService *self);
 
-void                 gcal_weather_service_run                       (GcalWeatherService *self,
-                                                                     GWeatherLocation   *location);
+void                 gcal_weather_service_activate                  (GcalWeatherService *self);
+
+void                 gcal_weather_service_deactivate                (GcalWeatherService *self);
+
+void                 gcal_weather_service_start                     (GcalWeatherService *self);
 
 void                 gcal_weather_service_stop                      (GcalWeatherService *self);
 
+void                 gcal_weather_service_hold                      (GcalWeatherService *self);
+
+void                 gcal_weather_service_release                   (GcalWeatherService *self);
+
 G_END_DECLS
 
 #endif /* GCAL_WEATHER_SERVICE_H */
